using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEngine;
using ConsoleLSystem;
using System;

public class Turtle3DAnimated : TurtleLSystem {
   public GameObject bud;
    public GameObject stem;
    public GameObject petal;
    public float angle;

    private Func<float[], Matrix4x4> _roation(Vector3 axis) {
        Matrix4x4 f(float[] args) {
            if (args.Length == 0) {
                return Matrix4x4.Rotate(Quaternion.AngleAxis(angle, axis));
            }
            else {
                return Matrix4x4.Rotate(Quaternion.AngleAxis(args[0], axis));
            }
        }
        return f;
    }

    protected override void initLiteralInterpretation() {
        turtleInterpretation = new Dictionary<string, Func<float[], Tuple<GameObject, Matrix4x4>>>();
        //turtleInterpretation
        
        turtleInterpretation.Add("B", (float[] args) => new Tuple<GameObject, Matrix4x4>(bud, Matrix4x4.identity));
        turtleInterpretation.Add("S", (float[] args) => new Tuple<GameObject, Matrix4x4>(stem, Matrix4x4.identity));
        turtleInterpretation.Add("P", (float[] args) => new Tuple<GameObject, Matrix4x4>(petal, Matrix4x4.Scale(Vector3.one*0.5f)));

        var transformation = Matrix4x4.Translate(new Vector3(0.0f, 0.2f, 0)) * Matrix4x4.Scale(new Vector3 (0.05f, 0.1f, 0.05f));

        turtleInterpretation.Add("+", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.back)(args) ));
        turtleInterpretation.Add("-", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.forward)(args)));

        turtleInterpretation.Add("\\", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.down)(args)));
        turtleInterpretation.Add("/", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.up)(args)));

        turtleInterpretation.Add("^", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.left)(args)));
        turtleInterpretation.Add("&", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, _roation(Vector3.right)(args)));


        turtleInterpretation.Add("f", (float[] args) => new Tuple<GameObject, Matrix4x4>(null, Matrix4x4.Translate(Vector3.up * args[0])));

    }
   
}
    